#ifdef GL_ES
precision highp float;
#endif

varying vec2 vTex;
uniform sampler2D samplerFront;

uniform float uDistance;
uniform float uAngle;
uniform float uOpacity;
uniform float uBlurRadius;

void main(void) {
    // Convert the angle to radians
    float angleRad = uAngle * 3.14159265 / 180.0;

    // Calculate the shadow offset based on the viewport size
    vec2 offset = uDistance * 0.001 * vec2(cos(angleRad), sin(angleRad));

    // Multi-sample blur
    vec4 shadow = vec4(0.0);
    const int numSamples = 5;  // number of samples; increase for better quality
    for (int i = -numSamples; i <= numSamples; ++i) {
        for (int j = -numSamples; j <= numSamples; ++j) {
            vec2 sampleCoord = vTex + offset + uBlurRadius * 0.0001 * vec2(float(i) / float(numSamples), float(j) / float(numSamples));
            shadow += texture2D(samplerFront, sampleCoord);
        }
    }
    shadow /= float((2*numSamples + 1)*(2*numSamples + 1));  // Average the samples

    shadow.rgb = vec3(0.0, 0.0, 0.0);  // Hard-coded black shadow color
    shadow.a *= uOpacity;

    // Get the original color
    vec4 col = texture2D(samplerFront, vTex);

    // Combine shadow with the sprite
    vec4 shadowColor = mix(col, vec4(0.0, 0.0, 0.0, shadow.a), shadow.a); 

    // Final color
    gl_FragColor = mix(shadowColor, col, col.a);
}